#!/usr/bin/env python
# coding: utf-8

# # File format example
# 0  :  
# 1  :  "TESTER","Series 4000","0"
# 2  :  "DATE","03/11/2021"
# 3  :  "TIME","11:28:36"
# 4  :  "USERFIELD1","Sensors-W121_12%, 18CN"
# 5  :  "USERFIELD2","Strips QC"
# 6  :  "USERFIELD3","820"
# 7  :  "USERFIELD4",""
# 8  :  "USERFIELD5",""
# 9  :  "USERFIELD6",""
# 10  :  "USERFIELD7",""
# 11  :  "USERFIELD8",""
# 12  :  "USERFIELD9",""
# 13  :  "USERFIELD10",""
# 14  :  "OPERATOR","Liam Boynton"
# 15  :  "LOADUNIT","g"
# 16  :  "CARTRIDGE","25 g","2073721","PULL(rotate)"
# 17  :  "TESTSPEED",1786.0,"um/s"
# 18  :  "TESTLOAD",5.00,"g"
# 19  :  "FALLBACK",30,"%"
# 20  :  "TEST",1,"3",12.707,""
# 21  :  "TEST",2,"1",12.075,""
# 22  :  "TEST",3,"1",11.905,""
# 23  :  "TEST",4,"1",11.620,""
# 24  :  "TEST",5,"1",11.959,""
# 
# 
# # Date formate
# UK format
# "now": "2021-11-24T15:29:08.641849+00:00"

# ## Append at top of list:
# https://stackoverflow.com/questions/17911091/append-integer-to-beginning-of-list-in-python
# 
# -Use this to take care of pull-tests without the qutation marks by checking list after reading and eventually adding an empty line at the top to not upset the rest of the code

# In[ ]:


import os
import sys
import argparse
import json
from glob import glob

from pathlib import Path
import datetime
import statistics





def analyse (data_file, db_ID, alternative_ID, component_type, component_stage, run_number, out_file_name):
    """This function analyses Liverpool pull-test csv files and
    creates a ITk DB json file for the PULL_TEST
    Output file name can be set by calling function"""
    
    print('Start csv file analysis')
    print(f"Data file: {data_file}\n")
    
      
        
    if out_file_name : print(f"Output file name {out_file_name}")

    
    #########Read File########################################
    with open(data_file, 'r') as d_file:
        lines = d_file.readlines()
        #lines = [line.rstrip() for line in lines]  #strip newlines and spaces at end of line
        lines = [line.rstrip().split(',') for line in lines]  #strip newlines and spaces at end of line, split into lists
    
    
    #print(lines)
    #print(lines[26][3])
    
   
    
    #########Analyse results######################################
    
    
    date_input = lines[2][1].lstrip('\"').rstrip('\"')
    time_input = lines[3][1].lstrip('\"').rstrip('\"')
    mydate = datetime.datetime.strptime(date_input,'%d/%m/%Y')
    mytime = datetime.datetime.strptime(time_input,'%H:%M:%S').time()

    datetimeobject = datetime.datetime.combine(mydate, mytime)
    #print(datetimeobject.astimezone().isoformat())
    
    operator = lines[14][1].lstrip('\"').rstrip('\"')
    #print(operator)
    pull_test_machine =lines[1][1].lstrip('\"').rstrip('\"')
    bond_machine = lines[6][1].lstrip('\"').rstrip('\"')
    
    #set from file if not pre-set by user
    if not db_ID :
        db_ID = str(''.join(lines[4][1:len(lines[4])]).lstrip('\"').rstrip('\"') )
        #db_ID = lines[4][1].lstrip('\"').rstrip('\"')
    if not component_type :
        component_type = lines[9][1].lstrip('\"').rstrip('\"')
    if not component_stage :
        component_stage = lines[10][1].lstrip('\"').rstrip('\"')
    
    
    #scan through all pull tests
    pull_grade = []
    pull_force = []
    minimum_force = 100.0
    maximum_force =0.0
    too_low = 0
    heel_breaks = 0
    
    
    for l in range(20,len(lines)):
        #print(lines[l])
        if lines[l][2].lstrip('\"').rstrip('\"') == 'E' : continue
        pull_force.append( float(lines[l][3]) )
        pull_grade.append( int(lines[l][2].lstrip('\"').rstrip('\"')) )
        if float(lines[l][3]) < minimum_force : minimum_force = float(lines[l][3])
        if float(lines[l][3]) > maximum_force : maximum_force = float(lines[l][3])
        if float(lines[l][3]) < 5 : too_low += 1
        if int(lines[l][2].lstrip('\"').rstrip('\"')) == 4 or int(lines[l][2].lstrip('\"').rstrip('\"')) == 5 :
            heel_breaks += 1
        
    heel_breaks_perc = ( 1 - float(heel_breaks)/float(len(pull_force)) ) * 100
    
    #print(pull_grade)
    #print(pull_force)
    #print(f"Minimum Force {minimum_force}" )
    #print(f"Maximum Force {maximum_force}" )
    #print(f"Too low {too_low}" )
    #print(f"Heel break {heel_breaks}" )
    #print(f"Heel break percent {heel_breaks_perc}" )
    
    if len(pull_grade) != len(pull_force): print("Pull Force read error")
    
    
    
    if heel_breaks_perc >= 90         and too_low == 0         and statistics.mean(pull_force) >= 8         and statistics.stdev(pull_force) <= 1.5 :
            passed = True
    else:
        passed = False
    
    #print( f"Passed: {passed}" )
    
    
   
    

    
    #########make plots###########################################
    
  
    
    
    #########create dictionary for json file######################
    out_dictionary = {}  ##dictionary for output
    
    out_dictionary.update({"componentType": component_type}) 
    out_dictionary.update({"component": db_ID})             
    out_dictionary.update({"alternativeIdentifier": alternative_ID})    
    out_dictionary.update({"currentStage": component_stage})        
    out_dictionary.update({"testType" : "PULL_TEST"})
    out_dictionary.update({"institution" : "LIV"})
    out_dictionary.update({"runNumber" : run_number})                           
    out_dictionary.update({"date" : datetimeobject.astimezone().isoformat(timespec='milliseconds')})      
    out_dictionary.update({"passed" : passed})
    out_dictionary.update({"problems" : False})
    out_dictionary.update({"properties":{
        "OPERATOR" : operator,                                      
        "NUMBER_WIRES" : int(len(pull_force)),                             
        "PULL_TEST_MACHINE" : pull_test_machine,                               
        "WIRE_BOND_MACHINE" : bond_machine                                     
    }  })
    out_dictionary.update({"results" :{
        "MEAN" : round( statistics.mean(pull_force),3),
        "STANDARD_DEVIATION" : round(statistics.stdev(pull_force),3),
        "TOO_LOW" : too_low, 
        "MINIMUM_STRENGTH" : round(minimum_force,3),
        "MAXIMUM_STRENGTH" : round(maximum_force,3),
        "HEEL_BREAKS" : heel_breaks_perc,
        "PULL_STRENGTH" : pull_force,     
        "PULL_GRADE" : pull_grade,               
        #"FILE" :  "./" + Path(data_file).name                     
        "FILE" : data_file
    }  })
    
    out_dictionary.update({"comments" : [ str(''.join(lines[4][1:len(lines[4])]).lstrip('\"').rstrip('\"') ) ] })
    
    
    
    #########save json file#######################################
    
    df_p = Path(data_file) ##datafile path
    
    #optional output filename or default from csv
    if out_file_name :
        out_file = str(df_p.parent) +'\\' + out_file_name + '.json'
    else :
        out_file = str(df_p.parent) + '\\' + db_ID + '_' + str(run_number) + '.json' 
    print(f"Json output file: {out_file}")
    

    
    with open( out_file, 'w' ) as outfile:
        json.dump(out_dictionary, outfile, indent=4)  #indent makes it human readable




# In[ ]:


if __name__ == "__main__":
    
    # Parse command line
    parser = argparse.ArgumentParser(description = "Script for analysing pull test csv files")
    parser.add_argument("csv", help = "Full name of csv file containing pull test results or directory of csv files")
    parser.add_argument("--dbID", default = None, help = "Database Component ID")
    parser.add_argument("--alternativeID", default = False, help = "Is alternative ID used? True or False")
    parser.add_argument("--componentType", default = None, help = "Component type (database code) ")
    parser.add_argument("--componentStage", default = None, help = "Component stage (database code) ")
    parser.add_argument("--runNumber", default = 0, help = "Run Number for test" )
    parser.add_argument("--outFileName", default = None, help = "Filename for json output.")
    args = parser.parse_args()
    
    
    #!python pull_test_reader Test.csv --dbID 20USB2940 --alternativeID True \
    #        --componentType HYBRID_FLEX --componentStage NEW --runNumber 5 --outFileName Test-New
    
    
    # Opening file
    
    
    
    #analyse (data_file, db_ID, alternative_ID, component_type, component_stage, run_number, out_file_name)
    if args.csv.endswith(".csv"):
        print(f"\n>>> Analysing {args.csv} ...")
        analyse(args.csv, args.dbID, args.alternativeID, args.componentType,                args.componentStage, args.runNumber, args.outFileName)          
    elif os.path.isdir(args.csv):
        files = glob(f"{args.csv}/*.csv")
        if not files:
            sys.exit(f"ERROR: No csv files found in directory {args.csv}")
        for f in files:
            print(f"\n>>> Analysing {f} ...")
            analyse(f, args.dbID, args.alternativeID, args.componentType,                args.componentStage, args.runNumber, args.outFileName)               
    else:
        sys.exit("ERROR: Unknown input format. Please provide a csv file or directory path containing csv files")


# In[ ]:




